package com.icl.saxon.expr;
import com.icl.saxon.*;
import com.icl.saxon.om.*;
import com.icl.saxon.axis.*;

import org.xml.sax.SAXException;
import java.util.*;

/**
* An expression of the form "//name"
*/

public class AllElementsExpression extends NodeSetExpression {

    private int fingerprint;
    private DocumentInfo boundRoot = null;

    /**
    * Constructor
    * @param name The required element name
    */

    public AllElementsExpression(int fingerprint) {
        this.fingerprint = fingerprint;
    }

    /**
    * Simplify an expression
    * @return the simplified expression
    */

    public Expression simplify() throws SAXException {
        return this;
    }

    /**
    * Determine which aspects of the context the expression depends on. The result is
    * a bitwise-or'ed value composed from constants such as Context.VARIABLES and
    * Context.CURRENT_NODE
    */

    public int getDependencies() {
        return (boundRoot==null ? Context.CONTEXT_NODE : 0 );
    }

    /**
    * Perform a partial evaluation of the expression, by eliminating specified dependencies
    * on the context.
    * @param dependencies The dependencies to be removed
    * @param context The context to be used for the partial evaluation
    * @return a new expression that does not have any of the specified
    * dependencies
    */

    public Expression reduce(int dependencies, Context context) throws SAXException {
        if (boundRoot==null && ((dependencies & Context.CONTEXT_NODE) != 0)) {
            AllElementsExpression aee = new AllElementsExpression(fingerprint);
            aee.boundRoot = context.getContextNode().getDocumentRoot();
            return aee;
        } else {
            return this;
        }
    }


    /**
    * Evaluate the path-expression in a given context to return a NodeSet
    * @param context the evaluation context
    * @param sort true if the returned nodes must be in document order
    */

    public NodeEnumeration enumerate(Context context, boolean sort) throws SAXException {   
        DocumentInfo root = (boundRoot==null ?
                             context.getContextNode().getDocumentRoot() :
                             boundRoot);
        return root.getAllElements(fingerprint).enumerate(); 
    }

    /**
    * Return a string representation of the expression
    * Used for diagnostics, and also for reconstituting the expression 
    */

    public String toString() {
        return "//{" + fingerprint + "}";
    }

}



//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/ 
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License. 
//
// The Original Code is: all this file. 
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (Michael.Kay@icl.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved. 
//
// Contributor(s): none. 
//
