package com.icl.saxon.om;
import com.icl.saxon.output.Outputter;

import java.util.Vector;
//import org.xml.sax.Attributes;
import org.xml.sax.SAXException;


/**
  * A node in the XML parse tree representing an XML element.<P>
  * The ElementInfo provides information about the element and its context.
  * Information available includes the tag and attributes of the element, and
  * pointers to the parent element and the previous element at the same level.<P>
  * @author <A HREF="mailto:Michael.Kay@icl.com>Michael H. Kay, ICL</A> 
  * @version 17 June 1999 - becomes an interface
  */

public interface ElementInfo extends NodeInfo {

    
    /**
    * Get the attribute list for this element.
    * @return The attribute list (as in the SAX interface). If you are using the Distributor and
    * you have not called setOptions(RETAIN_ATTRIBUTES) and this is not the startElement() call
    * for the element, the attribute list will be null.
    */
    
    public ExtendedAttributes getAttributeList();
       
    /**
    * Make an attribute node for a given attribute of this element
    * @param index The relative position of the attribute, numbered from 0..n-1 
    */

    public AttributeInfo makeAttributeNode(int index) throws SAXException;

    /**
    * Make an attribute node for a given attribute of this element
    * @param fingerprint The fingerprint of the attribute name 
    */

    public AttributeInfo makeAttributeNodeFS(int fingerprint) throws SAXException;

   /**
    * Search the NamespaceList for a given prefix, returning the corresponding URI code.
    * @param prefix The prefix to be matched. To find the default namespace, supply ""
    * @return The URI code corresponding to this namespace. If it is an unnamed default namespace,
    * return Namespace.NULL_CODE.
    * @throws SAXException if the prefix has not been declared on this element or a containing
    * element.
    */

    public short getURICodeForPrefix(String prefix) throws SAXException;

    /**
    * Search the NamespaceList for a given URI, returning the corresponding prefix.
    * @param uri The URI to be matched. To find the default namespace, supply ""
    * @return The prefix corresponding to this URI. If not found, return null.
    */

    public String getPrefixForURI(String uri);

    /**
    * Make the set of all namespace nodes associated with this element.
    * @param owner The element node to own these namespace nodes
    * @param list a vector containing NamespaceInfo objects representing the namespaces
    * in scope for this element; the method appends nodes to this Vector, which should
    * initially be empty.
    * @param addXML Add the XML namespace node to the list
    */

    public void addNamespaceNodes(ElementInfo owner, Vector list, boolean addXML) throws SAXException;

    /**
    * Output all namespace nodes associated with this element.
    * @param out The relevant outputter
    * @param includeAncestors True if namespaces declared on ancestor elements must
    * be output; false if it is known that these are already on the result tree
    */

    public void outputNamespaceNodes(Outputter out, boolean includeAncestors)
        throws SAXException;
    
}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/ 
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License. 
//
// The Original Code is: all this file. 
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (Michael.Kay@icl.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved. 
//
// Contributor(s): none. 
//
