package com.icl.saxon.om;

/**
  * The Sequencer class is responsible for allocating unique blocks of sequence numbers
  * used for numbering nodes. Sequence numbers allocated by one sequencer instance
  * are guaranteed unique.
  */
  
public class Sequencer

{
    private int nextSequenceNumber = 0;
    private static Sequencer defaultDocumentSequencer = new Sequencer();
    private static Sequencer defaultNodeSequencer = new Sequencer();

    /**
    * Allocate a block of sequence numbers
    * @param size the number of sequence numbers to be allocated
    * @return the first number in the allocated block
    */

    public synchronized int allocate(int size) {
        int a = nextSequenceNumber;
        nextSequenceNumber += size;
        return a;
    }

    /**
    * Get the default document sequencer
    */

    public static Sequencer getDefaultDocumentSequencer() {
        return defaultDocumentSequencer;
    }

    /**
    * Get the default node sequencer
    */

    public static Sequencer getDefaultNodeSequencer() {
        return defaultNodeSequencer;
    }    
}   

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/ 
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License. 
//
// The Original Code is: all this file. 
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (Michael.Kay@icl.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved. 
//
// Contributor(s): none. 
//
