package com.icl.saxon.output;
import com.icl.saxon.*;
//import com.icl.saxon.om.Name;
import com.icl.saxon.om.Namespace;
import com.icl.saxon.om.NamePool;

import org.xml.sax.*;
import org.w3c.dom.*;

import java.io.Writer;


/**
  * DOMEmitter is an Emitter that attaches the result tree to a specified Node in a DOM Document
  */
  
public class DOMEmitter implements Emitter
{
    protected OutputDetails outputDetails;
    protected Node currentNode;
    protected Document document;
    protected NamePool namePool;
    private boolean canNormalize = true;

	/**
	* Set the name pool to be used for all name codes
	*/
	
	public void setNamePool(NamePool pool) {
		namePool = pool;
	}

    /**
    * Set the Writer to use. Provided merely to satisfy the interface.
    */

    public void setWriter(Writer writer) {}

    /**
    * Set the character set to use. Provided merely to satisfy the interface.
    */

    public void setCharacterSet(CharacterSet charSet) {}

    /**
    * Set Document Locator. Provided merely to satisfy the interface.
    */

    public void setDocumentLocator(Locator locator) {}   

    /**
    * Start of the document. 
    */
    
    public void startDocument () throws SAXException 
    {

    }

    /**
    * End of the document. 
    */
    
    public void endDocument () throws SAXException
    {

    }


    /**
    * Start of an element. Output the start tag, escaping special characters.
    */
    
    public void startElement (int nameCode, Attributes attributes,
    						  int[] namespaces, int nscount) throws SAXException
    {
        String name = namePool.getDisplayName(nameCode);
        try {

            Element element = document.createElement(name);
            currentNode.appendChild(element);
            currentNode = element;

            // output the namespaces

            for (int n=0; n<nscount; n++) {
            	String prefix = namePool.getPrefixFromNamespaceCode(namespaces[n]);
        		String uri = namePool.getURIFromNamespaceCode(namespaces[n]);
                if (!(uri.equals(Namespace.XML))) {
                    if (prefix.equals("")) {
                        element.setAttribute("xmlns", uri);
                    } else {
                        element.setAttribute("xmlns:" + prefix, uri);
                    }
                }
            }

            // output the attributes

            for (int i=0; i<attributes.getLength(); i++) {
                element.setAttribute(
                    attributes.getQName(i),
                    attributes.getValue(i));
            }
            
        } catch (DOMException err) {
            throw new SAXException(err);
        }            
    }
    
    /**
    * End of an element.
    */

    public void endElement (int nameCode) throws SAXException
    {

		if (canNormalize) {
	        try {
	            currentNode.normalize();
	        } catch (Throwable err) {
	        	canNormalize = false;
	        }      // in case it's a Level 1 DOM
	    }
        
        currentNode = currentNode.getParentNode();

    }


    /**
    * Character data.
    */

    public void characters (char[] ch, int start, int length) throws SAXException
    {   
        try {
            Text text = document.createTextNode(new String(ch, start, length));
            currentNode.appendChild(text);
        } catch (DOMException err) {
            throw new SAXException(err);
        }            
    }


    /**
    * Handle a processing instruction.
    */
    
    public void processingInstruction (String target, String data)
        throws SAXException
    {
        try {
            ProcessingInstruction pi =
                document.createProcessingInstruction(target, data);
            currentNode.appendChild(pi);
        } catch (DOMException err) {
            throw new SAXException(err);
        }
    }

    /**
    * Handle a comment.
    */
    
    public void comment (char ch[], int start, int length) throws SAXException
    {
        try {
            Comment comment = document.createComment(new String(ch, start, length));
            currentNode.appendChild(comment);
        } catch (DOMException err) {
            throw new SAXException(err);
        }
    }

    /**
    * Set output details
    */
    
    public void setOutputDetails (OutputDetails details) {
        outputDetails = details;
        currentNode = details.getDOMNode();
        if (currentNode instanceof Document) {
            document = (Document)currentNode;
        } else {
            document = currentNode.getOwnerDocument();
        }
    }

    /**
    * Set escaping on or off: ignored in this Emitter
    */

    public void setEscaping(boolean escaping) {}

    /**
    * Set the URI for an unparsed entity in the document.
    */

    public void setUnparsedEntity(String name, String uri) throws SAXException {
	}
	
}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/ 
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License. 
//
// The Original Code is: all this file. 
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (Michael.Kay@icl.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved. 
//
// Contributor(s): none. 
//
