package com.icl.saxon.pattern;
import com.icl.saxon.om.NodeInfo;
//import com.icl.saxon.om.Name;
import com.icl.saxon.om.NamePool;
import org.xml.sax.SAXException;

/**
  * NodeTest is an interface that enables a test of whether a node has a particular
  * name and type. A NameTest matches the node type and the namespace URI and the local
  * name.
  *
  * @author <A HREF="mailto:Michael.Kay@icl.com>Michael H. Kay, ICL</A> 
  */

public class NameTest extends NodeTest {
	
	private short nodeType;
	private int fingerprint;
	
	public NameTest(short nodeType, int nameCode) {
		this.nodeType = nodeType;
		this.fingerprint = nameCode & 0xfffff;
	}

	/**
	* Create a NameTest for nodes of the same type and name as a given node
	*/
	
	public NameTest(NodeInfo node) {
		this.nodeType = node.getNodeType();
		this.fingerprint = node.getFingerprint();
	}

    /**
    * Test whether this node test is satisfied by a given node
    */

    public final boolean matches(NodeInfo node) {
    	return nodeType == node.getNodeType() && 
    		   fingerprint == node.getFingerprint();
    }

    /**
    * Determine the default priority of this node test when used on its own as a Pattern
    */

    public final double getDefaultPriority() {
    	return 0.0;
    }

	/**
	* Get the fingerprint required
	*/
	
	public int getFingerprint() {
		return fingerprint;
	}

    /**
    * Determine the types of nodes to which this pattern applies. Used for optimisation.
    * For patterns that match nodes of several types, return NodeInfo.NODE
    * @return the type of node matched by this pattern. e.g. NodeInfo.ELEMENT or NodeInfo.TEXT
    */

    public short getNodeType() {
        return nodeType;
    }

}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/ 
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License. 
//
// The Original Code is: all this file. 
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (Michael.Kay@icl.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved. 
//
// Contributor(s): none. 
//
