package com.icl.saxon.style;
import com.icl.saxon.*;
import com.icl.saxon.expr.*;
import org.xml.sax.SAXException;
import com.icl.saxon.om.ExtendedAttributes;
import java.util.*;

/**
* An xsl:namespace-alias element in the stylesheet.<BR>
*/

public class XSLNamespaceAlias extends StyleElement {

    private short stylesheetURICode;
    private short resultURICode;

    public void prepareAttributes() throws SAXException {

	    String stylesheetPrefix=null;
	    String resultPrefix=null;

		StandardNames sn = getStandardNames();
		ExtendedAttributes atts = getAttributeList();
		
		for (int a=0; a<atts.getLength(); a++) {
			int nc = atts.getNameCode(a);
			int f = nc & 0xfffff;
			if (f==sn.STYLESHEET_PREFIX) {
        		stylesheetPrefix = atts.getValue(a);
        	} else if (f==sn.RESULT_PREFIX) {
        		resultPrefix = atts.getValue(a);
        	} else {
        		checkUnknownAttribute(nc);
        	}
        }
        if (stylesheetPrefix==null) {
            reportAbsence("stylesheet-prefix");
        }
        if (stylesheetPrefix.equals("#default")) {
            stylesheetPrefix="";
        }
        if (resultPrefix==null) {
            reportAbsence("result-prefix");
        }
        if (resultPrefix.equals("#default")) {
            resultPrefix="";
        }
        stylesheetURICode = getURICodeForPrefix(stylesheetPrefix);
        resultURICode = getURICodeForPrefix(resultPrefix);
    }

    public void validate() throws SAXException {
        checkTopLevel();
    }

    public void preprocess() throws SAXException {}

    public void process(Context c) {}

    public short getStylesheetURICode() {
        return stylesheetURICode;
    }

    public short getResultURICode() {
        return resultURICode;
    }

}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/ 
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License. 
//
// The Original Code is: all this file. 
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (Michael.Kay@icl.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved. 
//
// Contributor(s): none. 
//
