package com.icl.saxon.style;
import com.icl.saxon.*;
import com.icl.saxon.expr.*;
import com.icl.saxon.pattern.*;
import com.icl.saxon.om.*;
import org.xml.sax.SAXException;
import org.xml.sax.AttributeList;
import java.util.*;

/**
* An xsl:preserve-space or xsl:strip-space elements in stylesheet.<BR>
*/

public class XSLPreserveSpace extends StyleElement {

    private boolean preserve;
    private String elements;

    public void prepareAttributes() throws SAXException {
 
		StandardNames sn = getStandardNames();
		ExtendedAttributes atts = getAttributeList();

        preserve = (getFingerprint()==sn.XSL_PRESERVE_SPACE);
		
		for (int a=0; a<atts.getLength(); a++) {
			int nc = atts.getNameCode(a);
			int f = nc & 0xfffff;
			if (f==sn.ELEMENTS) {
        		elements = atts.getValue(a);
        	} else {
        		checkUnknownAttribute(nc);
        	}
        }
        if (elements==null)
            reportAbsence("elements");             
    }

    public void validate() throws SAXException {
        checkTopLevel();
    }

    public void preprocess() throws SAXException
    {
        // elements is a space-separated list of element names

        Stripper stripper = getPrincipalStyleSheet().getStripper();
        StringTokenizer st = new StringTokenizer(elements);
        while (st.hasMoreTokens()) {
            String s = st.nextToken();
            if (s.equals("*")) {
                stripper.addRule(new AnyNodeTest(), preserve, getPrecedence());
            } else if (s.endsWith(":*")) {
                String prefix = s.substring(0, s.length()-2);
                stripper.addRule(
                			new NamespaceTest(
	                				NodeInfo.ELEMENT,
	                				getURICodeForPrefix(prefix)),
                			preserve,
                			getPrecedence());
            } else {
            	stripper.addRule(
            				new NameTest(
            						NodeInfo.ELEMENT,
            						makeNameCode(s, true)),
            				preserve,
            				getPrecedence());
            }
        }
    }

    public void process(Context c) {}

}

//
// The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
// you may not use this file except in compliance with the License. You may obtain a copy of the
// License at http://www.mozilla.org/MPL/ 
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied.
// See the License for the specific language governing rights and limitations under the License. 
//
// The Original Code is: all this file. 
//
// The Initial Developer of the Original Code is
// Michael Kay of International Computers Limited (Michael.Kay@icl.com).
//
// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved. 
//
// Contributor(s): none. 
//
